#! /usr/bin/env python

""" Some python classes and routines for the OpenPandora
(c) Laurens Keek 2011

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
import evdev

class TouchScreen:
  """
  Keep track of the status of the touch screen.
  TODO: do updating in a thread
  """
  
  def __init__(self, filename='/dev/input/touchscreen0'):
    """
    Initialize tracking the touch screen from given event device.
    """
    self.filename = filename
    self.device = evdev.Device(filename)
    self.name = self.device.name
    self.rawx = 0.0 # Default values, until we read events
    self.rawy = 0.0
    self.rawpressure = -1.0
    self.x = 0.0
    self.y = 0.0
    self.pressure = 0.0
    self.button = 0
    self.set_calibration_data()
  
  def set_calibration_data(self):
    """
    Set the data needed for the raw2*() routines to return calibrated
    values of the device output.
    """
    self.minpressure = 1.0 # Pressure limits, roughly from a quick test
    self.maxpressure = -0.1
  
  def update(self):
    """
    Update the status of the touch screen.
    """
    dev = self.device
    dev.poll()
    
    if dev.buttons.has_key('BTN_TOUCH'): # Mouse button emulation
      self.button = dev.buttons['BTN_TOUCH']
    
    if dev.axes.has_key('ABS_X'): # X coordinate
      self.rawx = dev.axes['ABS_X']
      self.x = self.raw2x(self.rawx)
    
    if dev.axes.has_key('ABS_Y'): # Y coordinate
      self.rawy = dev.axes['ABS_Y']
      self.y = self.raw2y(self.rawy)
    
    if dev.axes.has_key('ABS_PRESSURE'): # Pressure
      self.rawpressure = dev.axes['ABS_PRESSURE']
      self.pressure = self.raw2pressure(self.rawpressure)
      
  def raw2x(self, x):
    """
    Convert a raw x coordinate to a calibrated value.
    """
    return x
  
  def raw2y(self, y):
    """
    Convert a raw x coordinate to a calibrated value.
    """
    return y
  
  def raw2pressure(self, pressure):
    """
    Convert a raw x coordinate to a calibrated value.
    """
    if pressure == -1.0:
      return 0.0
    pressure = max(pressure, self.maxpressure)
    pressure = min(pressure, self.minpressure)
    pressure = (self.minpressure - pressure)/(self.minpressure - self.maxpressure)
    return pressure

if __name__ == '__main__':
  # Example usage:
  ts = TouchScreen()
  while True:
    ts.update()
    print 'X: %f; Y: %f; Pressure: %f; Button: %i'%(ts.x, ts.y, ts.pressure, ts.button)
    
